import platform
from megalus.core.utils import run_command
from buzio import formatStr

if platform.system().lower() == "windows":
    from win32api import *
    from win32gui import *
    import win32con
    import sys
    import os
    import struct
    import time


class WindowsBalloonTip:
    def __init__(self, title, msg):
        message_map = {
            win32con.WM_DESTROY: self.OnDestroy,
        }
        # Register the Window class.
        wc = WNDCLASS()
        hinst = wc.hInstance = GetModuleHandle(None)
        wc.lpszClassName = "PythonTaskbar"
        wc.lpfnWndProc = message_map  # could also specify a wndproc.
        classAtom = RegisterClass(wc)
        # Create the Window.
        style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
        self.hwnd = CreateWindow(
            classAtom,
            "Taskbar",
            style,
            0,
            0,
            win32con.CW_USEDEFAULT,
            win32con.CW_USEDEFAULT,
            0,
            0,
            hinst,
            None)
        UpdateWindow(self.hwnd)
        iconPathName = os.path.abspath(
            os.path.join(sys.path[0], "balloontip.ico"))
        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
        try:
            hicon = LoadImage(hinst, iconPathName,
                              win32con.IMAGE_ICON, 0, 0, icon_flags)
        except BaseException:
            hicon = LoadIcon(0, win32con.IDI_APPLICATION)
        flags = NIF_ICON | NIF_MESSAGE | NIF_TIP
        nid = (self.hwnd, 0, flags, win32con.WM_USER + 20, hicon, "tooltip")
        Shell_NotifyIcon(NIM_ADD, nid)
        Shell_NotifyIcon(NIM_MODIFY,
                         (self.hwnd, 0, NIF_INFO, win32con.WM_USER + 20,
                          hicon, "Balloon  tooltip", title, 200, msg))
        # self.show_balloon(title, msg)
        time.sleep(10)
        DestroyWindow(self.hwnd)

    def OnDestroy(self, hwnd, msg, wparam, lparam):
        nid = (self.hwnd, 0)
        Shell_NotifyIcon(NIM_DELETE, nid)
        PostQuitMessage(0)  # Terminate the app.


def notify(msg, title=None):
    if not title:
        title = "Megalus"
    if platform.system().lower() == 'linux':  # Linux
        run_command(
            task='notify-send -i gsd-xrandr {title} {msg}'.format(
                title='"{}"'.format(formatStr.unitext(title)),
                msg='"{}"'.format(formatStr.unitext(msg))
            )
        )
    elif platform.system().lower() == "windows" \
            and platform.version() == "10":  # Windows 10
        
        WindowsBalloonTip(msg, title)

    else:  # Mac
        run_command(
            task="osascript -e 'display notification "
            "{msg} with title {title}'".format(
                title='"{}"'.format(
                    formatStr.unitext(title)),
                msg='"{}"'.format(
                    formatStr.unitext(msg))
            )
        )
